import React, { FC } from 'react';
import styled from 'styled-components';

import {
  blenderBackground,
  blenderForeground,
  getBlenderTypographyType,
} from '@clds/blender';
// eslint-disable-next-line no-duplicate-imports
import type { BlenderProps, BlendMode, BlendVariant } from '@clds/blender';
import { globalTheme } from '@clds/component-theme';
import { Label } from '@clds/dev-components';
import { typography, Typography } from '@clds/typography';

import { chess, Grid, useConfigPanel } from './preview';

export default {
  title: 'Blender',
};

const ExampleRoot = styled.div`
  background-color: ${blenderBackground};
  display: inline-block;
  padding: ${globalTheme.spacing.sm};
  border-radius: ${globalTheme.radius.lg};
`;

const ExampleDot = styled.div`
  background-color: ${blenderForeground};
  display: inline-block;
  margin-right: ${globalTheme.spacing.sm};
  border-radius: ${globalTheme.radius.unstable_new.xl};
  width: ${globalTheme.spacing.sm};
  height: ${globalTheme.spacing.sm};
  vertical-align: middle;
`;

const Example: FC<BlenderProps> = ({ children, ...rest }) => (
  <ExampleRoot {...rest}>
    <ExampleDot {...rest} />
    <Typography
      as="span"
      size="sm"
      type={getBlenderTypographyType(rest.mode, rest.variant)}
    >
      {children}
    </Typography>
  </ExampleRoot>
);

const HBox = styled.div`
  display: flex;
  flex-direction: row;
`;
const VBox = styled.div`
  display: flex;
  flex-direction: column;
`;

const BlenderColumn: FC<{ mode: BlendMode; variant: BlendVariant }> = ({
  mode,
  variant,
}) => (
  <>
    <Example intensity="sm" variant={variant} mode={mode}>
      {mode} sm
    </Example>
    <Example intensity="md" variant={variant} mode={mode}>
      {mode} md
    </Example>
    <Example intensity="lg" variant={variant} mode={mode}>
      {mode} lg
    </Example>
    <Example intensity="xl" variant={variant} mode={mode}>
      {mode} xl
    </Example>
  </>
);

const AmbientBlenderColumn: FC<{ mode: BlendMode; variant: BlendVariant }> = ({
  mode,
  variant,
}) => (
  <>
    <AmbientBackground variant={variant}>
      <Example intensity="sm" variant={variant} mode={mode}>
        {mode} sm
      </Example>
    </AmbientBackground>
    <AmbientBackground variant={variant}>
      <Example intensity="md" variant={variant} mode={mode}>
        {mode} md
      </Example>
    </AmbientBackground>
    <AmbientBackground variant={variant}>
      <Example intensity="lg" variant={variant} mode={mode}>
        {mode} lg
      </Example>
    </AmbientBackground>
    <AmbientBackground variant={variant}>
      <Example intensity="xl" variant={variant} mode={mode}>
        {mode} xl
      </Example>
    </AmbientBackground>
  </>
);

const AmbientBackground = styled.div<{ variant: BlendVariant }>`
  ${(props) =>
    chess(
      blenderBackground({ ...props, intensity: 'md', mode: 'solidContrast' }),
      blenderBackground({ ...props, intensity: 'lg', mode: 'solidContrast' }),
    )};
  align-self: stretch;
  justify-self: stretch;
  display: grid;
  align-items: stretch;
  justify-content: stretch;
  padding: ${globalTheme.spacing.xxs};
  ${ExampleRoot} {
    padding: ${globalTheme.spacing.xs};
  }
`;

const Legend = styled.div`
  h4 {
    ${typography.sm.highContrast};
    margin: ${globalTheme.spacing.md} 0 ${globalTheme.spacing.xs} 0;
  }
  div {
    ${typography.xs.regular};
  }
  code {
    padding: ${globalTheme.spacing.xxs};
    background: ${globalTheme.palette.surfaceAlt};
    border-radius: ${globalTheme.radius.unstable_new.md};
  }
`;

export const Default = () => {
  const { panel, variant, preview } = useConfigPanel();
  return (
    <HBox>
      {panel}
      <VBox>
        <Label>
          This grid contains example components using Cloudinary Blender. In
          columns you can see different background blending (plain, surface,
          solid). In rows - blending intensity (sm, md, lg, xl).
        </Label>
        <Grid grid={preview}>
          <BlenderColumn mode="plainContrast" variant={variant} />
          <BlenderColumn mode="surfaceContrast" variant={variant} />
          <BlenderColumn mode="plainSolid" variant={variant} />
          <BlenderColumn mode="surfaceSolid" variant={variant} />
          <BlenderColumn mode="solidContrast" variant={variant} />
          <AmbientBlenderColumn mode="plainInvert" variant={variant} />
        </Grid>
        <Legend>
          <h4>Plain Contrast</h4>
          <div>
            Surfaces with background starting from <code>transparent</code> (md)
            and going a bit into the variant color. Foreground color is{' '}
            <code>contrastHigh</code>.
          </div>
          <h4>Surface Contrast</h4>
          <div>
            Surfaces with background starting from <code>surface</code> (md) and
            going a bit into the variant color. Foreground color is{' '}
            <code>contrastHigh</code>.
          </div>
          <h4>Plain Solid</h4>
          <div>
            Surfaces with background starting from <code>transparent</code> (md)
            and going a bit into the variant color. Foreground color is solid{' '}
            <code>variant</code>.
          </div>
          <h4>Surface Solid</h4>
          <div>
            Surfaces with background starting from <code>surface</code> (md) and
            going a bit into the variant color. Foreground color is solid{' '}
            <code>variant</code>.
          </div>
          <h4>Solid Contrast</h4>
          <div>
            Surfaces with background starting from solid <code>variant</code>{' '}
            (md) and going a bit into the darker/lighter color. Foreground color
            is usually <code>contrastInvert</code>.
          </div>
          <h4>Plain Invert</h4>
          <div>
            Surfaces with background starting from <code>transparent</code> (md)
            and going a bit into the darker/lighter color. Foreground is usually{' '}
            <code>contrastInvert</code>.
          </div>
          <div>
            <strong>
              This is special blend mode to be used on a surface that is
              surrounded by <code>solidContrast</code> surface.
            </strong>
          </div>
        </Legend>
      </VBox>
    </HBox>
  );
};
